@@ -1,7 +1,7 @@
 package Data::Cmp;
 
-our $DATE = '2018-08-12'; # DATE
-our $VERSION = '0.004'; # VERSION
+our $DATE = '2018-08-13'; # DATE
+our $VERSION = '0.005'; # VERSION
 
 use 5.010001;
 use strict;
@@ -59,13 +59,11 @@ sub _cmp_data {
         $_seen_refaddrs{$refaddr1}++;
         $_seen_refaddrs{$refaddr2}++;
       ELEM:
-        for my $i (0..$#{$d1}) {
-            if ($i > $#{$d2}) { return 1 }
+        for my $i (0..($#{$d1} < $#{$d2} ? $#{$d1} : $#{$d2})) {
             my $cmpres = _cmp_data($d1->[$i], $d2->[$i]);
             return $cmpres if $cmpres;
         }
-        if (@$d2 > @$d1) { return -1 }
-        return 0;
+        return $#{$d1} <=> $#{$d2};
     } elsif ($reftype1 eq 'HASH' && !$_seen_refaddrs{$refaddr1} && !$_seen_refaddrs{$refaddr2}) {
         $_seen_refaddrs{$refaddr1}++;
         $_seen_refaddrs{$refaddr2}++;
@@ -106,7 +104,7 @@ Data::Cmp - Compare two data structures, return -1/0/1 like cmp
 
 =head1 VERSION
 
-This document describes version 0.004 of Data::Cmp (from Perl distribution Data-Cmp), released on 2018-08-12.
+This document describes version 0.005 of Data::Cmp (from Perl distribution Data-Cmp), released on 2018-08-13.
 
 =head1 SYNOPSIS
 
@@ -132,10 +130,10 @@ Sort data structures (of similar structures):
 
 =head1 DESCRIPTION
 
-This relatively compact module offers the C<cmp_data> function that, like Perl's
-C<cmp>, returns -1/0/1 value. In addition to that, it can also return 2 if the
-two data structures differ but there is no sensible notion of which one is
-"greater than" the other.
+This relatively lightweight (no non-core dependencies, under 100 lines of code)
+module offers the C<cmp_data> function that, like Perl's C<cmp>, returns -1/0/1
+value. In addition to that, it can also return 2 if the two data structures
+differ but there is no sensible notion of which one is "greater than" the other.
 
 This module can handle circular structure.
 
@@ -169,7 +167,7 @@ The following are the rules of comparison used by C<cmp_data()>:
  cmp_data(bless([], "foo"), bless([], "bar")); # 2
  cmp_data(bless([], "foo"), bless([], "foo")); # 0
 
-=item * Two array references are compared element by element
+=item * Two array references are compared element by element (unless an array has been seen, see last rule)
 
  cmp_data(["a","b","c"], ["a","b","c"]); #  0
  cmp_data(["a","b","c"], ["a","b","d"]); # -1
@@ -179,7 +177,7 @@ The following are the rules of comparison used by C<cmp_data()>:
 
  cmp_data(["a","b"], ["a"]); # 1
 
-=item * Two hash references are compared key by key
+=item * Two hash references are compared key by key (unless an array has been seen, see last rule)
 
  cmp_data({k1=>"a", k2=>"b", k3=>"c"}, {k1=>"a", k2=>"b", k3=>"d"}); # 0
  cmp_data({k1=>"a", k2=>"b", k3=>"c"}, {k1=>"a", k2=>"b", k3=>"c"}); # 1
@@ -192,6 +190,19 @@ If the number of non-common pairs are the same, they are just different.
  cmp_data({k1=>"", k2=>"", k3=>""}, {k1=>"", k5=>"", k6=>"});         #  2
  cmp_data({k1=>"", k2=>"", k3=>""}, {k1=>"", k5=>"", k6=>", k7=>""}); # -1
 
+=item * All other types of references (i.e. non-hash, non-array) are the same only if their address is
+
+ cmp_data(\1, \1); # 2
+ my $ref = \1; cmp_data($ref, $ref); # 0
+
+=item * A seen (hash or array) reference is no longer recursed, it's compared by address (see previous rule)
+
+ my $ary1 = [1]; push @$ary1, $ary1;
+ my $ary2 = [1]; push @$ary2, $ary2;
+ my $ary3 = [1]; push @$ary3, $ary1;
+ cmp_data($ary1, $ary2); # 1
+ cmp_data($ary1, $ary3); # 0
+
 =back
 
 =head1 FUNCTIONS
