@@ -1,7 +1,7 @@
 package Data::Cmp::StrOrNumeric;
 
-our $DATE = '2018-08-12'; # DATE
-our $VERSION = '0.004'; # VERSION
+our $DATE = '2018-08-13'; # DATE
+our $VERSION = '0.005'; # VERSION
 
 use 5.010001;
 use strict;
@@ -71,13 +71,12 @@ sub _cmp_data {
         $_seen_refaddrs{$refaddr1}++;
         $_seen_refaddrs{$refaddr2}++;
       ELEM:
-        for my $i (0..$#{$d1}) {
+        for my $i (0..($#{$d1} < $#{$d2} ? $#{$d1} : $#{$d2})) {
             if ($i > $#{$d2}) { return 1 }
             my $cmpres = _cmp_data($d1->[$i], $d2->[$i]);
             return $cmpres if $cmpres;
         }
-        if (@$d2 > @$d1) { return -1 }
-        return 0;
+        return $#{$d1} <=> $#{$d2};
     } elsif ($reftype1 eq 'HASH' && !$_seen_refaddrs{$refaddr1} && !$_seen_refaddrs{$refaddr2}) {
         $_seen_refaddrs{$refaddr1}++;
         $_seen_refaddrs{$refaddr2}++;
@@ -118,7 +117,7 @@ Data::Cmp::StrOrNumeric - Compare two data structures, return -1/0/1 like cmp or
 
 =head1 VERSION
 
-This document describes version 0.004 of Data::Cmp::StrOrNumeric (from Perl distribution Data-Cmp), released on 2018-08-12.
+This document describes version 0.005 of Data::Cmp::StrOrNumeric (from Perl distribution Data-Cmp), released on 2018-08-13.
 
 =head1 SYNOPSIS
 
